@@ -1,11 +1,13 @@
 __version__ = "0.1.0"
 
 
+from .custom_scalar import scalar  # noqa
 from .enum import enum  # noqa
 from .field import field  # noqa
 from .mutation import mutation, subscription  # noqa
 from .scalars import ID  # noqa
 from .schema import Schema  # noqa
 from .type import input, type, interface  # noqa
+from .types import *  # noqa
 from .permission import BasePermission  # noqa
 from .directive import directive  # noqa
