@@ -12,11 +12,13 @@ DIRECTIVE_REGISTRY = {}
 
 def _get_arguments(func):
     annotations = func.__annotations__
-    name = func.__name__
 
     arguments = {}
 
     for name, type_ in islice(annotations.items(), 1, None):
+        if name == 'return':
+            continue
+
         argument_type = get_graphql_type_for_annotation(type_, name)
 
         name = to_camel_case(name)
