@@ -1,13 +1,18 @@
+import dataclasses
 from functools import partial
 
-import dataclasses
 from graphql import GraphQLInputObjectType, GraphQLInterfaceType, GraphQLObjectType
-from graphql.utilities.schema_printer import print_type
 
 from .constants import IS_STRAWBERRY_FIELD, IS_STRAWBERRY_INPUT, IS_STRAWBERRY_INTERFACE
 from .field import field, strawberry_field
 from .type_converter import REGISTRY
 from .utils.str_converters import to_camel_case
+from .utils.typing import get_actual_type
+
+
+def _interface_resolve_type(result, info, return_type):
+    """Resolves the correct type for an interface"""
+    return result.__class__.field
 
 
 def _get_resolver(cls, field_name):
@@ -17,7 +22,10 @@ def _get_resolver(cls, field_name):
         return class_field.resolver
 
     def _resolver(root, info):
-        field_resolver = getattr(root or cls(), field_name, None)
+        if not root:
+            return None
+
+        field_resolver = getattr(root, field_name, None)
 
         if getattr(field_resolver, IS_STRAWBERRY_FIELD, False):
             return field_resolver(root, info)
@@ -35,29 +43,29 @@ def _process_type(cls, *, is_input=False, is_interface=False, description=None):
     name = cls.__name__
     REGISTRY[name] = cls
 
-    def repr_(self):
-        return print_type(self.field)
-
-    setattr(cls, "__repr__", repr_)
-
-    def _get_fields(wrapped):
+    def _get_fields(wrapped, types_replacement_map=None):
         class_fields = dataclasses.fields(wrapped)
 
         fields = {}
 
         for class_field in class_fields:
+            class_field.type = get_actual_type(class_field.type, types_replacement_map)
+
             field_name = getattr(class_field, "field_name", None) or to_camel_case(
                 class_field.name
             )
             description = getattr(class_field, "field_description", None)
-
+            permission_classes = getattr(class_field, "field_permission_classes", None)
             resolver = getattr(class_field, "field_resolver", None) or _get_resolver(
                 cls, class_field.name
             )
             resolver.__annotations__["return"] = class_field.type
 
             fields[field_name] = field(
-                resolver, is_input=is_input, description=description
+                resolver,
+                is_input=is_input,
+                description=description,
+                permission_classes=permission_classes,
             ).field
 
         strawberry_fields = {}
@@ -85,10 +93,17 @@ def _process_type(cls, *, is_input=False, is_interface=False, description=None):
 
     extra_kwargs = {"description": description or cls.__doc__}
 
+    wrapped = dataclasses.dataclass(cls)
+
     if is_input:
         TypeClass = GraphQLInputObjectType
     elif is_interface:
         TypeClass = GraphQLInterfaceType
+
+        # TODO: in future we might want to be able to override this
+        # for example to map a class (like a django model) to one
+        # type of the interface
+        extra_kwargs["resolve_type"] = _interface_resolve_type
     else:
         TypeClass = GraphQLObjectType
 
@@ -98,8 +113,11 @@ def _process_type(cls, *, is_input=False, is_interface=False, description=None):
             if hasattr(klass, IS_STRAWBERRY_INTERFACE)
         ]
 
-    wrapped = dataclasses.dataclass(cls, repr=False)
-    wrapped.field = TypeClass(name, lambda: _get_fields(wrapped), **extra_kwargs)
+    wrapped.field = TypeClass(
+        name,
+        lambda types_replacement_map=None: _get_fields(wrapped, types_replacement_map),
+        **extra_kwargs
+    )
 
     return wrapped
 
